// Tales of Syzygies and Transformations

// let's play with cyclic(4) again

LIB "poly.lib";
LIB "primdec.lib";

option(redSB);
option(redTail);

ring r = 0,(x,y,z,t),dp;

ideal I = cyclic(4); I;

ideal J = groebner(I);

dim(J); // not 0-dimensional

// are elements of Groebner basis minimal as generators of an ideal?

module S = syz(J); // syzygies

print(S); // not much can be seen explicitly yet

print(S[2]); // hence we access vectors separately

// let us write the procedure extracting all vectors which have a constant entry!

proc findConstants(module M)
{
  int nc = ncols(M); 
  int nr = nrows(M); 
  int i,j; 
  module N;
  for(i=1; i<= nc; i++)
  {
    for(j=1; j<= nr; j++)
    {
      if (M[i][j] != 0)
      {
        if (leadexp(M[i][j]) == 0 )
        {
          N = N, M[i];
        }
      }
    }
  }
  N = N[2..size(N)];
  return(N);
}

module N =  findConstants(S); 

// and some analysis implies there will be 4 elements


// From F. Winkler's lecture, Example 7.1.

LIB "poly.lib"; // for normalize

ring W = 0,(x,y,z),(dp,C);

ideal F = x*z - x*y2 - 4*x2 -1/4, y2*z + 2*x + 1/2, x2*z + y2 + 1/2*x;

option(prot); option(redSB); option(redTail);

module S = syz(F);

print(S); 

S = groebner(S); S = normalize(S); // unique GB of S

// now let us see the admissible presentations

ideal G = std(F); // Groebner basis. Note that no el't of F are in G!

poly f = F[1];

print(lift(G,f)); // cofactors via the lift matrix

// or cofactors via module elimination trick

vector ff = f*gen(1); // that is f is put into the 1st component

module GG = G[1]*gen(1) + gen(2), G[2]*gen(1) + gen(3), G[3]*gen(1) + gen(4), G[4]*gen(1) + gen(5);

// write a procedure for an automatic generation of such!

attrib(GG,"isSB",1);

vector rr = NF(ff,GG);

print(-rr); // the cofactors

quit;
